package com.yoho.opsmanager.other.handlder;

import com.yoho.opsmanager.other.clientReport.ClientPatchRunnable;
import com.yoho.opsmanager.other.clientReport.ClientWriteRunnable;
import com.yoho.opsmanager.other.influxdb.InfluxDBWriter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PreDestroy;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.*;

/**
 *  APP 上报消息处理，包括错误处理、补丁上报处理
 * Created by chunhua.zhang on 2017/5/24.
 */
@Service
public class AppReportProcessor {

    private final static Logger log = LoggerFactory.getLogger(AppReportProcessor.class);

    @Autowired
    private InfluxDBWriter writer;

    /**
     * 50个消息处理一次
     */
    private final int BATCH_SIZE = 50;
    /**
     * 临时存储客户端上报的消息，大小为 1000
     */
    private final static LinkedBlockingQueue<ErrorInfoHolder> APP_ERROR = new LinkedBlockingQueue<>(1000);

    /**
     * 设备类型
     */
   public enum DeviceType{
        ios,
        android
    }

    /**
     *  新的错误上报处理
     */
    private final ExecutorService executorService;

    /**
     * 先兼容之前的处理方式
     */
    @Deprecated
    private final ExecutorService preService;

    /**
     * 构造2个处理器
     */
    public AppReportProcessor(){
        executorService = Executors.newFixedThreadPool(10);
        preService = Executors.newFixedThreadPool(10);
    }


    /**
     * 处理错误
     * @param msg 错误消息
     * @param devType 设备类型
     */
    public void submitError(String msg, DeviceType devType){
        try {
            APP_ERROR.put(new ErrorInfoHolder(msg,devType));
            //clear queue and add to list
            List<ErrorInfoHolder>  batchMsgs = new LinkedList<>();
            if(APP_ERROR.size() >= BATCH_SIZE){
                APP_ERROR.drainTo(batchMsgs);
            }
            //process
            this.executorService.submit(new AppErrorReportHandler(writer,batchMsgs));

        } catch (InterruptedException e) {
            log.error("error when processing mes:" + msg, e);
        }


        //兼容之前的处理方式
        preService.execute(new ClientWriteRunnable(msg,devType.name()));
    }

    /**
     * 处理补丁上报事件
     * @param msg 补丁消息
     * @param deviceType 设备类型
     */
    public void submitPatch(String msg, DeviceType deviceType){
        preService.submit(new ClientPatchRunnable(msg, deviceType.name()));
    }


    @PreDestroy
    public void destory(){
        executorService.shutdown();
        preService.shutdown();
    }

    final static class ErrorInfoHolder{

        public final String msg;
        public final DeviceType deviceType;

        ErrorInfoHolder(String msg, DeviceType deviceType) {
            this.msg = msg;
            this.deviceType = deviceType;
        }
    }



 }

